﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Telerik.Windows.Controls;

namespace RibbonTest
{
    /// <summary>
    /// Interaction logic for UserControl1.xaml
    /// </summary>
    public partial class ColorSelector : UserControl
    {

        #region events

        public static readonly RoutedEvent ColorChangedEvent;

        #endregion



        public ColorSelector()
        {

            InitializeComponent();
        }



        static ColorSelector()
        {

            //===== Dependency properties

            SelectedColorProperty = DependencyProperty.Register(

                        "SelectedColor", typeof(Color), typeof(ColorSelector),

                        new FrameworkPropertyMetadata(Colors.Transparent, new PropertyChangedCallback(SelectedColorChanged)));





            RedProperty = DependencyProperty.Register(

                        "Red", typeof(byte), typeof(ColorSelector),

                        new FrameworkPropertyMetadata(new PropertyChangedCallback(OnColorRGBChanged)));

            GreenProperty = DependencyProperty.Register(

                        "Green", typeof(byte), typeof(ColorSelector),

                        new FrameworkPropertyMetadata(new PropertyChangedCallback(OnColorRGBChanged)));

            BlueProperty = DependencyProperty.Register(

                        "Blue", typeof(byte), typeof(ColorSelector),

                        new FrameworkPropertyMetadata(new PropertyChangedCallback(OnColorRGBChanged)));

            RGBSelectedColorProperty = DependencyProperty.Register(

                        "RGBSelectedColor", typeof(Color), typeof(ColorSelector));



            //===== events

            ColorChangedEvent = EventManager.RegisterRoutedEvent("ColorChanged", RoutingStrategy.Bubble,

                typeof(RoutedPropertyChangedEventHandler<Color>), typeof(ColorSelector));

        }



        #region events

        public event RoutedPropertyChangedEventHandler<Color> ColorChanged
        {

            add { AddHandler(ColorChangedEvent, value); }

            remove { RemoveHandler(ColorChangedEvent, value); }

        }

        private void OnColorChanged(Color oldValue, Color newValue)
        {

            RoutedPropertyChangedEventArgs<Color> args = new RoutedPropertyChangedEventArgs<Color>(oldValue, newValue);

            args.RoutedEvent = ColorSelector.ColorChangedEvent;

            RaiseEvent(args);

        }

        #endregion



        #region SelectedColor

        public Color SelectedColor
        {

            get { return (Color)GetValue(SelectedColorProperty); }

            set { SetValue(SelectedColorProperty, value); }

        }

        private static void SelectedColorChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {

            ColorSelector ob = d as ColorSelector;

            Color value = (Color)e.NewValue;

            Color oldColor = (Color)e.OldValue;

            if (ob != null)
            {

                ob.OnColorChanged(oldColor, value);

                ob.Red = value.R;

                ob.Green = value.G;

                ob.Blue = value.B;

                ob.RGBSelectedColor = value;

                //try

                //{

                //    object oParent = ob.Parent;

                //    // Tentative de récupération d'un Popup parent du Ctrl_ColorSelector

                //    while (oParent != null && !(oParent is Popup) && oParent is FrameworkElement)

                //        oParent = ((FrameworkElement)oParent).Parent;

                //    if (oParent != null && oParent is Popup)

                //        // Fermeture du Popup parent du Ctrl_ColorSelector

                //        ((Popup)oParent).IsOpen = false;

                //}

                //catch { }

            }

        }

        #endregion









        #region RGB Colors

        public Color RGBSelectedColor
        {

            get { return (Color)GetValue(RGBSelectedColorProperty); }

            set { SetValue(RGBSelectedColorProperty, value); }

        }

        public byte Red
        {

            get { return (byte)GetValue(RedProperty); }

            set { SetValue(RedProperty, value); }

        }

        public byte Green
        {

            get { return (byte)GetValue(GreenProperty); }

            set { SetValue(GreenProperty, value); }

        }

        public byte Blue
        {

            get { return (byte)GetValue(BlueProperty); }

            set { SetValue(BlueProperty, value); }

        }



        private static void OnColorRGBChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {

            ColorSelector ob = sender as ColorSelector;

            Color color = ob.RGBSelectedColor;

            if (color.A == 0) color.A = Convert.ToByte("FF", 16);



            if (e.Property == RedProperty) color.R = (byte)e.NewValue;

            else if (e.Property == GreenProperty) color.G = (byte)e.NewValue;

            else if (e.Property == BlueProperty) color.B = (byte)e.NewValue;



            ob.RGBSelectedColor = color;

        }



        //private bool _bApplyclicked;

        private void Apply_Click(object sender, RoutedEventArgs e)
        {

            this.SelectedColor = this.RGBSelectedColor;



            //int timestamp = new System.TimeSpan(System.DateTime.Now.Ticks).Milliseconds;

            //System.Windows.Input.MouseButton mouseButton = System.Windows.Input.MouseButton.Left;



            //System.Windows.Input.MouseButtonEventArgs ClickEvent = new System.Windows.Input.MouseButtonEventArgs(System.Windows.Input.Mouse.PrimaryDevice, timestamp, mouseButton);

            //ClickEvent.RoutedEvent = Control.MouseDownEvent;

            //ClickEvent.Source = tabcontrol;

            //_bApplyclicked = true;

            //tabcontrol.RaiseEvent(ClickEvent);



            //timestamp = new System.TimeSpan(System.DateTime.Now.Ticks).Milliseconds;

            //ClickEvent = new System.Windows.Input.MouseButtonEventArgs(System.Windows.Input.Mouse.PrimaryDevice, timestamp, mouseButton);

            //ClickEvent.RoutedEvent = Control.MouseUpEvent;

            //ClickEvent.Source = tabcontrol;

            //tabcontrol.RaiseEvent(ClickEvent); 

        }

        private void MousedownHandled(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {

            if (tabcontrol.SelectedIndex == 1) e.Handled = true;

            //_bApplyclicked = false;

        }





        private void test(object sender, SelectionChangedEventArgs e)
        {

            if (e.OriginalSource == tabcontrol)

                e.Handled = true;

        }

        #endregion



        #region StaticFields

        public static readonly DependencyProperty SelectedColorProperty;

        public static readonly DependencyProperty RedProperty;

        public static readonly DependencyProperty GreenProperty;

        public static readonly DependencyProperty BlueProperty;

        public static readonly DependencyProperty RGBSelectedColorProperty;



        #endregion

        private void RadColorSelector_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {

        }





    }

}